
#define IDENTIFICATION "1997.03.17 SHOWTIME 2.11 <ailanto@neosplice.com>"

/*
**
**   Authors:
**
**      William W. Patterson
**
**   History:
**
**      1995.04.12   0.00   WWP   Creation
**      1995.04.12   0.01   WWP   Do julian day number
**      1995.04.12   0.02   WWP   Added -u for use this date
**      1995.04.12   0.03   WWP   Determined MINDATE & MAXDATE for compiler
**      1995.04.12   0.04   WWP   Added -d & -t
**      1995.04.12   0.05   WWP   Added -f
**      1995.04.12   0.06   WWP   Added -l
**      1995.04.12   0.07   WWP   Added -w
**      1995.04.12   1.00   WWP   Cut! Print!
**      1995.04.18   1.01   WWP   Show date with day number, as in VMS version
**      1995.04.28   1.02   WWP   Use -b for both instead of -f
**      1995.04.28   1.03   WWP   Let options be case-sensitive for more choices
**      1995.04.28   1.04   WWP   Replaced -l with -D
**      1995.04.28   1.05   WWP   Added -W to show weekday etc.
**      1995.05.02   1.06   WWP   Expand date range via bigjulian()
**      1995.05.02   1.07   WWP   Old -j case sensitive, added -J
**      1995.05.02   1.08   WWP   Added -v for debugging
**      1995.05.02   1.09   WWP   Within valid cc range, compare results
**      1995.05.02   2.00   WWP   Cut! Print!
**      1995.05.02   2.01   WWP   Added calendar function
**      1995.12.06   2.02   WWP   Changed email address
**      1996.07.02   2.10   WWP   Release!
**      1997.03.17   2.11   WWP   Changed email address
**
**   Notes:
**
**      See help() for usage
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>

/*
**   for reference purposes. this is defined in time.h
**   valid from 1980.01.01 through 2038.01.18
**
**   struct tm {
**      int tm_sec;     seconds after the minute - [0,59]
**      int tm_min;     minutes after the hour - [0,59]
**      int tm_hour;    hours since midnight - [0,23]
**      int tm_mday;    day of the month - [1,31]
**      int tm_mon;     months since January - [0,11]
**      int tm_year;    years since 1900
**      int tm_wday;    days since Sunday - [0,6]
**      int tm_yday;    days since January 1 - [0,365]
**      int tm_isdst;   daylight savings time flag
**      };
**
*/

char months[120] =
   "January\0  February\0 March\0    April\0    May\0      June\0     July\0     August\0   September\0October\0  November\0 December\0 ";

char days[70] =
   "Sunday\0   Monday\0   Tuesday\0  Wednesday\0Thursday\0 Friday\0   Saturday\0 ";

time_t tod;

struct tm *now;

struct tm poop;
struct tm *jnow = &poop;

long usedate = 0L;

/* if the valid range changes, be sure to check the calendar function! */

#define CMINDATE 19800101L
#define CMAXDATE 20380118L

#define JMINDATE 19000301L
#define JMAXDATE 21000228L

#define FALSE 0
#define TRUE 1

int verbose = FALSE;

int cvalid = TRUE;
int jvalid = FALSE;

/******************************************************************************/

#define GETOPTERR(s,c) \
   if (opterr) \
      { \
      char errbuf[2]; \
      errbuf[0] = c; \
      errbuf[1] = '\n'; \
      (void) write(2,argv[0],(unsigned)strlen(argv[0])); \
      (void) write(2,s,(unsigned)strlen(s)); \
      (void) write(2,errbuf,2); \
      }

int opterr = 1;
int optind = 1;
int optopt;
char *optarg;

int getopt( int argc, char **argv, char *opts )

   {

   static int sp = 1;
   register int c;
   register char *cp;

   if ( sp == 1 )
      {
      if ( optind>=argc || argv[optind][0]!='-' || argv[optind][1]=='\0' )
         {
         return(EOF);
         }
      else
         {
         if ( strcmp(argv[optind],"--") == (int)NULL ) /* 002 */
            {
            ++optind;
            return(EOF);
            }
         }
      }

   optopt = c = argv[optind][sp];

   if ( c==':' || (cp=strchr(opts,c))==NULL )
      {
      GETOPTERR(": illegal option -- ", c);
      if ( argv[optind][++sp] == '\0' )
         {
         ++optind;
         sp = 1;
         }
      return('?');
      }

   if ( *++cp == ':' )
      {
      if ( argv[optind][sp+1] != '\0' )
         {
         optarg = &argv[optind++][sp+1];
         }
      else
         {
         if ( ++optind >= argc )
            {
            GETOPTERR(": option requires an argument -- ",c);
            sp = 1;
            return('?');
            }
         else
            {
            optarg = argv[optind++];
            }
         }
      sp = 1;
      }
   else
      {
      if ( argv[optind][++sp] == '\0' )
         {
         sp = 1;
         ++optind;
         }
      optarg = NULL;
      }

   return(c);

   }

/****************************************************************************/

long julian( void )
   {
   return (long)( 1 + now->tm_yday );
   }

/*****************************************************************************
**
**   calculate full julian day number, days since 1 January -4713, using an
**   old algorithm that came from one of my calculator manuals
**
*****************************************************************************/

long bigjulian( long year, long month, long day )
   {
   year = month > 2 ? year : year - 1;
   month = month > 2 ? month + 1 : month + 13;
   return 1720982L + day + (long)(30.6001L*month) + (long)(365.25L*year);
   }

/*****************************************************************************
**
**   if we got this far, jvalid must be true! but cvalid might not be...
**   if both are true, do it both ways as a check
**
*****************************************************************************/

long setdate( long usedate )

   {

   long year;
   long month;
   long day;
   long jdn;
   long dow;
   long status = 0L;

   day = usedate % 100L;
   usedate /= 100L;
   month = usedate % 100L;
   usedate /= 100L;
   year = usedate;

   if ( cvalid )
      {
      if ( verbose ) printf("using standard c algorithm\n");
      now->tm_sec = 0;                       /* seconds after the minute - [0,59] */
      now->tm_min = 0;                       /* minutes after the hour - [0,59]   */
      now->tm_hour = 12;                     /* hours since midnight - [0,23]     */
      now->tm_mday = (int) day;              /* day of the month - [1,31]         */
      now->tm_mon = (int) ( month - 1 );     /* months since January - [0,11]     */
      now->tm_year = (int) ( year - 1900L ); /* years since 1900                  */
      /* now->tm_wday; */ /* mktime! */      /* days since Sunday - [0,6]         */
      /* now->tm_yday; */ /* mktime! */      /* days since January 1 - [0,365]    */
      now->tm_isdst = -1;                    /* daylight savings time flag        */
      status = mktime(now);
      }

   if ( jvalid )
      {
      if ( verbose ) printf("using alternate algorithm\n");
      jdn = bigjulian(year,month,day) - 1720982L;
      year = ( jdn - 122.1L ) / 365.25L;
      month = ( jdn - (long)( 365.25L * year ) ) / 30.6001L;
      day = jdn - (long)( 365.25L * year ) - (long)( 30.6001L * month );
      month = month < 14 ? month - 1 : month - 13;
      year = month > 2 ? year : year + 1;
      dow = ( jdn + 5 ) % 7;
      jnow->tm_sec = 0;
      jnow->tm_min = 0;
      jnow->tm_hour = 12;
      jnow->tm_isdst = -1;
      jnow->tm_year = (int) ( year - 1900 );
      jnow->tm_mon = (int) ( month - 1 );
      jnow->tm_mday = (int) day;
      jnow->tm_wday = (int) dow;
      jnow->tm_yday = (int) ( bigjulian(year,month,day) - bigjulian(year,1L,1L) );
      }

   if ( cvalid && jvalid )
      {
      if ( verbose )
         {
         printf("comparing results from the two algorithms (c,j)\n");
         printf("seconds after the minute - [0,59] %5d %5d\n", now->tm_sec, jnow->tm_sec);
         printf("minutes after the hour - [0,59]   %5d %5d\n", now->tm_min, jnow->tm_min);
         printf("hours since midnight - [0,23]     %5d %5d\n", now->tm_hour, jnow->tm_hour);
         printf("day of the month - [1,31]         %5d %5d\n", now->tm_mday, jnow->tm_mday);
         printf("months since January - [0,11]     %5d %5d\n", now->tm_mon, jnow->tm_mon);
         printf("years since 1900                  %5d %5d\n", now->tm_year, jnow->tm_year);
         printf("days since Sunday - [0,6]         %5d %5d\n", now->tm_wday, jnow->tm_wday);
         printf("days since January 1 - [0,365]    %5d %5d\n", now->tm_yday, jnow->tm_yday);
         }
      if ( now->tm_sec != jnow->tm_sec)   printf("tm_sec algorithmic disagreement\n");
      if ( now->tm_min != jnow->tm_min)   printf("tm_min algorithmic disagreement\n");
      if ( now->tm_hour != jnow->tm_hour) printf("tm_hour algorithmic disagreement\n");
      if ( now->tm_mday != jnow->tm_mday) printf("tm_mday algorithmic disagreement\n");
      if ( now->tm_mon != jnow->tm_mon)   printf("tm_mon algorithmic disagreement\n");
      if ( now->tm_year != jnow->tm_year) printf("tm_year algorithmic disagreement\n");
      if ( now->tm_wday != jnow->tm_wday) printf("tm_wday algorithmic disagreement\n");
      if ( now->tm_yday != jnow->tm_yday) printf("tm_yday algorithmic disagreement\n");
      }

   if ( !cvalid ) memcpy( now, jnow, sizeof ( struct tm ) );

   return status;

   }

/****************************************************************************/

void calendar( long usedate )
   {

   long year;
   long month;
   long day;

   long jdnfirst;
   long jdnlast;

   long firstday;
   long firstdow;

   int i;
   int dow;
   long jdn;

   day = usedate % 100L;
   usedate /= 100L;
   month = usedate % 100L;
   usedate /= 100L;
   year = usedate;

   firstday = ( 10000L * year ) + ( 100L * month ) + 1L;

   jdnfirst = bigjulian( year, month, 1 );

   if ( usedate == JMAXDATE )
      {
      jdnlast = bigjulian( year, month, day);
      }
   else
      {
      if ( month < 12 )
         {
         ++month;
         }
      else
         {
         month = 1;
         ++year;
         }
      jdnlast = bigjulian( year, month, 1 ) - 1;
      }

   setdate( firstday );

   firstdow = now->tm_wday;

   if ( verbose ) printf("%ld %ld-%ld\n", firstdow, jdnfirst, jdnlast );

   printf("\n  %s %04d\n", months + ( 10 * now->tm_mon ), 1900 + now->tm_year );

   printf("  Su  Mo  Tu  We  Th  Fr  Sa\n");

   for ( i = 0; i < firstdow; ++i ) printf("    ");

   for (
   i = 1, jdn = jdnfirst, dow = firstdow;
   jdn <= jdnlast;
   ++jdn, ++i, dow = (dow+1) % 7 )
      {
      printf("  %02d",i);
      if ( dow == 6 ) printf("\n");
      }

   if ( dow ) printf("\n");

   }

/****************************************************************************/

void help( void )
   {
   printf("\n%s\n\n",IDENTIFICATION);
   printf("usage: showtime [options]\n\n");
   printf("Options which do not have an uppercase partner are not case sensitive.\n\n");
   printf("   ? -? -h       help for program usage\n");
   printf("   -uyyyymmdd    use this date (at noon), range is [%ld..%ld]\n",JMINDATE,JMAXDATE);
   printf("   -j            julian day number, of the year\n");
   printf("   -J            full julian day number, days since 1 Jan -4713\n");
   printf("   -d            show date\n");
   printf("   -D            show date in long form\n");
   printf("   -t            show time\n");
   printf("   -b            show both date and time\n");
   printf("   -w            show day of week\n");
   printf("   -W            show day of week and date\n");
   printf("   -c            calendar for month containing selected day\n");
   printf("   -v            verbose, for debugging\n");
   }

/****************************************************************************/


main( int argc, char **argv )

   {

   int opt;

   time(&tod);
   now = localtime(&tod);

   if ( argc < 2 )
      {
      printf("\njulian day number for %02d %s %04d is %03ld\n", now->tm_mday, months + ( 10 * now->tm_mon ), 1900 + now->tm_year,julian());
      exit(0);
      }

   while ( optind < argc )
      {
      while ( ( opt = getopt(argc,argv,"?bBcCdDhHjJtTu:U:vVwW") ) != EOF )
         {
         switch(opt)
            {
            case 'b':
            case 'B':
               printf("\n");
               if (usedate) printf("[%ld] ",usedate);
               printf("%04d.%02d.%02d ", 1900 + now->tm_year, 1 + now->tm_mon, now->tm_mday );
               printf("%02d:%02d:%02d\n", now->tm_hour, now->tm_min, now->tm_sec );
               break;
            case 'c':
            case 'C':
               if (!usedate) usedate = 10000L * ( 1900 + now->tm_year ) + 100L * ( 1 + now->tm_mon ) + now->tm_mday;
               calendar(usedate);
               if ( 0 > setdate(usedate) )
                  {
                  printf("unable to setdate() for %ld\n",usedate);
                  exit(EINVAL);
                  }
               break;
            case 'd':
               printf("\n");
               if (usedate) printf("[%ld] ",usedate);
               printf("%04d.%02d.%02d\n", 1900 + now->tm_year, 1 + now->tm_mon, now->tm_mday );
               break;
            case 'D':
               printf("\n");
               if (usedate) printf("[%ld] ",usedate);
               printf("%02d %s %04d\n", now->tm_mday, months + ( 10 * now->tm_mon ), 1900 + now->tm_year );
               break;
            case 'h':
            case 'H':
            case '?':
               help();
               break;
            case 'j':
               printf("\n");
               if (usedate) printf("[%ld] ",usedate);
               printf("julian day number for %02d %s %04d is %03ld\n", now->tm_mday, months + ( 10 * now->tm_mon ), 1900 + now->tm_year,julian());
               break;
            case 'J':
               printf("\n");
               if (usedate) printf("[%ld] ",usedate);
               printf("days since 1 January -4713 is %ld\n", bigjulian( 1900 + now->tm_year, 1 + now->tm_mon, now->tm_mday ));
               break;
            case 't':
            case 'T':
               printf("\n");
               if (usedate) printf("[%ld] ",usedate);
               printf("%02d:%02d:%02d\n", now->tm_hour, now->tm_min, now->tm_sec );
               break;
            case 'u':
            case 'U':
               usedate = atol(optarg);
               cvalid = ( ( usedate >= CMINDATE ) && ( usedate <= CMAXDATE ) );
               jvalid = ( ( usedate >= JMINDATE ) && ( usedate <= JMAXDATE ) );
               if ( ! ( cvalid || jvalid ) )
                  {
                  printf("date %ld is out of range\n",usedate);
                  exit(EINVAL);
                  };
               if ( 0 > setdate(usedate) )
                  {
                  printf("unable to setdate() for %ld\n",usedate);
                  exit(EINVAL);
                  }
               break;
            case 'v':
            case 'V':
               verbose = TRUE;
               break;
            case 'w':
               printf("\n");
               if (usedate) printf("[%ld] ",usedate);
               printf("%s\n", days + ( 10 * now->tm_wday ) );
               break;
            case 'W':
               printf("\n");
               if (usedate) printf("[%ld] ",usedate);
               printf("%s, ", days + ( 10 * now->tm_wday ) );
               printf("%02d %s %04d\n", now->tm_mday, months + ( 10 * now->tm_mon ), 1900 + now->tm_year );
               break;
            default:
               help();
               break;
            }
         }
      if ( optind < argc )
         {
         switch(*argv[optind])
            {
            case '?':
               help();
               break;
            default:
               printf("unrecognized argument: %s\n",argv[optind]);
               help();
               break;
            }
         ++optind;
         }
      }
   }

